#ifndef SVN_VERSION_H
#define SVN_VERSION_H

#define SVN_VERSION "3422898505b74c637a4b530a94b63be90bdf025b [/startdir/boinc] (archlinux:/build/boinc/src/boinc [makepkg]) [Server-Release: server_release/1.x/1.x.0-3664-g3422898505b74c637a4b530a94b63be90bdf025b]"
#define GIT_REVISION 0x34228985
#define GIT_DATE 1781495868
#define REPOSITORY__STARTDIR_BOINC 1

#endif
